using ExternalDependency;
using Xunit;
using XunitExt;

namespace LegacyCode.Tests
{
    public class InlineStaticClassExample
    {
        [Fact]
        public void Should_not_allow_negative_tax_values()
        {
            decimal badTaxValue = -10.0m;
            var taxService = new StubTaxService(badTaxValue);
            FakeOrder order = new FakeOrder(taxService, new StubShippingCalculator(0));

            order.CalculateBaseSubTotal().ShouldEqual(order.CalculateBaseItemTotal());
        }

        private class StubTaxService : ITaxService
        {
            private readonly decimal _badTaxValue;

            public StubTaxService(decimal badTaxValue)
            {
                _badTaxValue = badTaxValue;
            }

            public decimal GetTaxAmount(Order order)
            {
                return _badTaxValue;
            }
        }

        private class StubShippingCalculator : ShippingCalculator
        {
            private readonly decimal _shippingCost;

            public StubShippingCalculator(decimal shippingCost)
            {
                _shippingCost = shippingCost;
            }

            public override decimal CalculateShippingCosts(Order order)
            {
                return _shippingCost;
            }
        }

        private class FakeOrder : Order
        {
            public FakeOrder(ITaxService taxService, ShippingCalculator shippingCalculator) 
                : base(taxService, shippingCalculator)
            {
            }

            // Needed to get to sub total calculation
            public decimal CalculateBaseSubTotal()
            {
                return base.CalculateSubTotal();
            }

            public decimal CalculateBaseItemTotal()
            {
                return base.GetItemTotal();
            }

        }
    }
}