using ExternalDependency;
using Xunit;
using XunitExt;

namespace LegacyCode.Tests
{
    public class ParameterizeConstructorExample
    {
        [Fact]
        public void Should_give_free_shipping_when_order_exceeds_fifty_dollars()
        {
            decimal shippingAmount = 20.0m;
            StubShippingCalculator calculator = new StubShippingCalculator(shippingAmount);
            ShoppingCart cart = new StubCart(70.0m, calculator);

            cart.CalculateDiscount();

            cart.DiscountAmount.ShouldEqual(shippingAmount);
        }

        private class StubShippingCalculator : ShippingCalculator
        {
            private readonly decimal _shippingCost;

            public StubShippingCalculator(decimal shippingCost)
            {
                _shippingCost = shippingCost;
            }

            public override decimal CalculateShippingCosts(Order order)
            {
                return _shippingCost;
            }
        }

        private class StubCart : ShoppingCart
        {
            private readonly decimal _subTotal;

            public StubCart(decimal subTotal, ShippingCalculator calculator) : base(calculator)
            {
                _subTotal = subTotal;
            }

            protected override decimal CalculateSubTotal()
            {
                return _subTotal;
            }
        }

    }
}