using ExternalDependency;

namespace LegacyCode
{
    public class ShoppingCart : Order
    {
        private readonly ShippingCalculator _shippingCalculator;

        // New no-arg constructor preserves existing behavior
        // we don't break existing clients, a good thing
        public ShoppingCart() : this (new ShippingCalculator())
        {
        }

        // New constructor gives us our seam - now we can put in fake ones
        public ShoppingCart(ShippingCalculator shippingCalculator)
        {
            _shippingCalculator = shippingCalculator;
        }

        protected override ShippingCalculator ShippingCalculator
        {
            get { return _shippingCalculator; }
        }

        public void CalculateDiscount()
        {
            if (this.CalculateSubTotal() > 100.0m)
                DiscountAmount = this.CalculateSubTotal()*0.10m;

            if (this.CalculateSubTotal() > 50.0m)
                DiscountAmount = _shippingCalculator.CalculateShippingCosts(this);
        }

        // Shadowed virtual method gives us our test seam
        // Now we can override the behavior - and existing behavior is preserved
        protected new virtual decimal CalculateSubTotal()
        {
            return base.CalculateSubTotal();
        }
    }
}